from .read.read_bam import *
from .read.read_h5 import *
from .write.write_h5 import *
from .write.write_text import *

__doc__="""\

IO API
============
.. currentmodule:: cfdna.io

Functions
---------
.. autosummary::
   :toctree: .

   read_bam
   read_h5
   write_h5
   write_seg

"""